/**
 * Raspberry Pi Pico - Voltmeter
 * Sends via uart
 * Works with ADC pins 0 to 4 
 * pins 0 to 2 are gp26 to gp28
 * pin 4 is internal temp sensor
 * See: www.penguintutor.com/projects/pico
 */


#include "pico/stdlib.h"
#include "hardware/uart.h"
#include "hardware/irq.h"
#include "hardware/adc.h"
#include "stdio.h"


/// \tag::voltmeter[]

#define UART_ID uart1
#define BAUD_RATE 115200
#define DATA_BITS 8
#define STOP_BITS 1
#define PARITY    UART_PARITY_NONE

// We are using pins 0 and 1, but see the GPIO function select table in the
// datasheet for information on which other pins can be used.
#define UART_TX_PIN 4
#define UART_RX_PIN 5

// Whch pin to monitor - default to 0
static int adc_pin = 0;

const float conversion_factor=3.3f/(1<<12);

// RX interrupt handler
void on_uart_rx() {
    char msg_string[20];
    while (uart_is_readable(UART_ID)) {
        uint8_t ch = uart_getc(UART_ID);
        // Is this number 0 to 4 (as ascii char)
        if (ch >= '0' && ch <= '4') {
            if (adc_pin != ch - '0') {
                adc_pin = ch - '0';
                adc_select_input(adc_pin);
            }
            
            uint16_t adc_value=adc_read();
            sprintf (msg_string, "Pin %d value %f\n", adc_pin, adc_value*conversion_factor);
            if (uart_is_writable(UART_ID)) {
                uart_puts(UART_ID, msg_string);
            }
        }
    }
}

int main() {
    // setup ADC
    stdio_init_all();
    adc_init();
    // set gpio as adc (no_pullups etc)
    adc_gpio_init(26);
    adc_gpio_init(27);
    adc_gpio_init(28);
    adc_select_input(adc_pin);

    // Set up our UART with a basic baud rate.
    uart_init(UART_ID, 2400);

    // Set the TX and RX pins by using the function select on the GPIO
    // Set datasheet for more information on function select
    gpio_set_function(UART_TX_PIN, GPIO_FUNC_UART);
    gpio_set_function(UART_RX_PIN, GPIO_FUNC_UART);

    // Actually, we want a different speed
    // The call will return the actual baud rate selected, which will be as close as
    // possible to that requested
    int actual = uart_set_baudrate(UART_ID, BAUD_RATE);

    // Set UART flow control CTS/RTS, we don't want these, so turn them off
    uart_set_hw_flow(UART_ID, false, false);

    // Set our data format
    uart_set_format(UART_ID, DATA_BITS, STOP_BITS, PARITY);

    // Turn off FIFO's - we want to do this character by character
    uart_set_fifo_enabled(UART_ID, false);

    // Set up a RX interrupt
    // We need to set up the handler first
    // Select correct interrupt for the UART we are using
    int UART_IRQ = UART_ID == uart0 ? UART0_IRQ : UART1_IRQ;

    // And set up and enable the interrupt handlers
    irq_set_exclusive_handler(UART_IRQ, on_uart_rx);
    irq_set_enabled(UART_IRQ, true);

    // Now enable the UART to send interrupts - RX only
    uart_set_irq_enables(UART_ID, true, false);

    // OK, all set up.
    // Run a loop and wait for RX interrupts
    while (1)
        tight_loop_contents();
}

/// \end:voltmeter[]
